@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23728 $
+ * $Revision: 28445 $
  * $Author: rjongbloed $
- * $Date: 2009-10-29 18:42:22 -0500 (Thu, 29 Oct 2009) $
+ * $Date: 2012-10-02 20:11:02 -0500 (Tue, 02 Oct 2012) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -114,6 +114,11 @@ class H460_FeatureSet;
 #endif
 
 
+/**OpalConnection::StringOption key to an . Default true.
+  */
+#define OPAL_OPT_Q931_BEARER_CAPS "Q931-Bearer-Caps"
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /**This class represents a particular H323 connection between two endpoints.
@@ -162,6 +167,9 @@ class H323Connection : public OpalRTPConnection
       */
     virtual PString GetPrefixName() const;
 
+    /// Call back for connection to act on changed string options
+    virtual void OnApplyStringOptions();
+
     /**Start an outgoing connection.
        This function will initiate the connection to the remote entity, for
        example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
@@ -220,7 +228,7 @@ class H323Connection : public OpalRTPConnection
        An application will not typically call this function as it is used by
        the OpalManager during a release of the connection.
 
-       The default behaviour calls CleanUpOnCallEnd() then calls the ancestor.
+       The default behaviour calls OnRelease() then calls the ancestor.
       */
     virtual void OnReleased();
 
@@ -260,6 +268,14 @@ class H323Connection : public OpalRTPConnection
       */
     virtual bool SetAlertingType(const PString & info);
 
+    /** Get Q.931 Progress Indicator IE description from received PROGRESS or ALERTING pdus.
+      */
+    unsigned GetProgressIndicator() const { return m_progressIndicator; }
+
+    /** Set Q.931 Progress Indicator IE description to send in PROGRESS or ALERTING pdus.
+      */
+    void SetProgressIndicator(unsigned indicator) { m_progressIndicator = indicator; }
+
     /**Get the data formats this connection is capable of operating.
        This provides a list of media data format names that an
        OpalMediaStream may be created in within this connection.
@@ -280,7 +296,7 @@ class H323Connection : public OpalRTPConnection
     /**Switch to/from FAX mode.
       */
     virtual bool SwitchFaxMediaStreams(
-      bool enableFax  ///< Enable FAX or return to audio mode
+      bool toT38  ///< Enable T.38 or return to audio mode
     );
 #endif
 
@@ -301,7 +317,7 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Get information on the media channel for the connection.
-       The default behaviour returns PTrue and fills the info structure if
+       The default behaviour returns true and fills the info structure if
        there is a media channel active for the sessionID.
      */
     virtual PBoolean GetMediaInformation(
@@ -310,34 +326,6 @@ class H323Connection : public OpalRTPConnection
     ) const;
   //@}
 
-  /**@name Backward compatibility functions */
-  //@{
-    /** Called when a connection is cleared, just after CleanUpOnCallEnd()
-        Default behaviour is to call H323Connection::OnConnectionCleared
-      */
-    virtual void OnCleared();
-
-    /**Determine if the call has been established.
-       This can be used in combination with the GetCallEndReason() function
-       to determine the three main phases of a call, call setup, call
-       established and call cleared.
-      */
-    PBoolean IsEstablished() const { return connectionState == EstablishedConnection; }
-
-    /**Clean up the call clearance of the connection.
-       This function will do any internal cleaning up and waiting on background
-       threads that may be using the connection object. After this returns it
-       is then safe to delete the object.
-
-       An application will not typically use this function as it is used by the
-       H323EndPoint during a clear call.
-      */
-    virtual void CleanUpOnCallEnd();
-
-    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
-  //@}
-
-
   /**@name Signalling Channel */
   //@{
     /**Attach a transport to this connection as the signalling channel.
@@ -377,7 +365,7 @@ class H323Connection : public OpalRTPConnection
        The default behaviour is to do the handshaking operation calling a few
        virtuals at certain moments in the sequence.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
      */
     virtual PBoolean OnReceivedSignalSetup(
@@ -385,7 +373,7 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Handle an incoming Q931 setup acknowledge PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour does nothing.
@@ -395,7 +383,7 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Handle an incoming Q931 information PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour does nothing.
@@ -405,31 +393,31 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Handle an incoming Q931 call proceeding PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks for hH245Address field and if present
        starts the separate H245 channel, if successful or not present it
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedCallProceeding(
       const H323SignalPDU & pdu   ///<  Received call proceeding PDU
     );
 
     /**Handle an incoming Q931 progress PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks for hH245Address field and if present
        starts the separate H245 channel, if successful or not present it
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedProgress(
       const H323SignalPDU & pdu   ///<  Received call proceeding PDU
     );
 
     /**Handle an incoming Q931 alerting PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour obtains the display name and calls OnAlerting().
@@ -439,51 +427,51 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Handle an incoming Q931 connect PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks for hH245Address field and if present
-       starts the separate H245 channel, if successful it returns PTrue.
-       If not present and there is no H245Tunneling then it returns PFalse.
+       starts the separate H245 channel, if successful it returns true.
+       If not present and there is no H245Tunneling then it returns false.
      */
     virtual PBoolean OnReceivedSignalConnect(
       const H323SignalPDU & pdu   ///<  Received connect PDU
     );
 
     /**Handle an incoming Q931 facility PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks for hH245Address field and if present
        starts the separate H245 channel, if successful or not present it
-       returns PTrue.
+       returns true.
      */
     virtual PBoolean OnReceivedFacility(
       const H323SignalPDU & pdu   ///<  Received connect PDU
     );
 
     /**Handle an incoming Q931 Notify PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnReceivedSignalNotify(
       const H323SignalPDU & pdu   ///<  Received connect PDU
     );
 
     /**Handle an incoming Q931 Status PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnReceivedSignalStatus(
       const H323SignalPDU & pdu   ///<  Received connect PDU
     );
 
     /**Handle an incoming Q931 Status Enquiry PDU.
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour sends a Q931 Status PDU back.
@@ -503,8 +491,8 @@ class H323Connection : public OpalRTPConnection
     /**This function is called from the HandleSignallingChannel() function
        for unhandled PDU types.
 
-       If PFalse is returned the connection is aborted and a Release Complete
-       PDU is sent. The default behaviour returns PTrue.
+       If false is returned the connection is aborted and a Release Complete
+       PDU is sent. The default behaviour returns true.
      */
     virtual PBoolean OnUnknownSignalPDU(
       const H323SignalPDU & pdu  ///<  Received PDU
@@ -564,7 +552,7 @@ class H323Connection : public OpalRTPConnection
        application to alter the reply before transmission to the other
        endpoint.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour calls the endpoint function of the same name.
@@ -579,7 +567,7 @@ class H323Connection : public OpalRTPConnection
        function when an application wishes to redirct an unwanted incoming
        call.
 
-       The return value is PTrue if the call is to be forwarded, PFalse
+       The return value is true if the call is to be forwarded, false
        otherwise. Note that if the call is forwarded the current connection is
        cleared with the ended call code of EndedByCallForwarded.
       */
@@ -598,24 +586,23 @@ class H323Connection : public OpalRTPConnection
     );
 
     /**Put the current connection on hold, suspending all media streams.
-     * Simply calls HoldCall() which is kept for backward compatibility.
-     */
-    virtual bool HoldConnection();
+       The \p fromRemote parameter indicates if we a putting the remote on
+       hold (false) or it is a request for the remote to put us on hold (true).
 
-    /**Retrieve the current connection from hold, activating all media 
-     * streams.
-     * Simply calls RetrieveCall() which is kept for backward compatibility.
+       The /p placeOnHold parameter indicates of teh command/request is for
+       going on hold or retrieving from hold.
      */
-    virtual bool RetrieveConnection();
+    virtual bool Hold(
+      bool fromRemote,  ///< Flag for if remote has us on hold, or we have them
+      bool placeOnHold  ///< Flag for setting on or off hold
+    );
 
     /**Return true if the current connection is on hold.
-       The bool parameter indicates if we are testing if the remote system
-       has us on hold, or we have them on hold.
-
-       Simply calls IsCallOnHold() which is kept for backward compatibility.
+       The \p fromRemote parameter indicates if we are testing if the remote
+       system has us on hold, or we have them on hold.
      */
-    virtual bool IsConnectionOnHold(
-      bool fromRemote  ///< Test if remote has us on hold, or we have them
+    virtual bool IsOnHold(
+      bool fromRemote  ///< Flag for if remote has us on hold, or we have them
     );
 
 #if OPAL_H450
@@ -763,7 +750,7 @@ class H323Connection : public OpalRTPConnection
        connection is created for Call Intrusion. This flag is used when we
        should decide whether to Answer the call or to Close it.
       */
-    void SetCallIntrusion() { isCallIntrusion = PTrue; }
+    void SetCallIntrusion() { isCallIntrusion = true; }
 
     PBoolean IsCallIntrusion() { return isCallIntrusion; }
 
@@ -868,7 +855,7 @@ class H323Connection : public OpalRTPConnection
        sends the Setup PDU. It gives an opportunity for an application to
        alter the request before transmission to the other endpoint.
 
-       The default behaviour simply returns PTrue. Note that this is usually
+       The default behaviour simply returns true. Note that this is usually
        overridden by the transport dependent descendent class, eg the
        H323ConnectionTCP descendent fills in the destCallSignalAddress field
        with the TCP/IP data. Therefore if you override this in your
@@ -882,9 +869,9 @@ class H323Connection : public OpalRTPConnection
        the OnReceivedSignalSetup() function before it sends the Call
        Proceeding PDU. It gives an opportunity for an application to alter
        the request before transmission to the other endpoint. If this function
-       returns PFalse then the Call Proceeding PDU is not sent at all.
+       returns false then the Call Proceeding PDU is not sent at all.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnSendCallProceeding(
       H323SignalPDU & callProceedingPDU   ///<  Call Proceeding PDU to send
@@ -894,12 +881,12 @@ class H323Connection : public OpalRTPConnection
        This allows an application to add things to the release complete before
        it is sent to the remote endpoint.
 
-       Returning PFalse will prevent the release complete from being sent. Note
+       Returning false will prevent the release complete from being sent. Note
        that this would be very unusual as this is called when the connection
        is being cleaned up. There will be no second chance to send the PDU and
        it must be sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
       */
     virtual PBoolean OnSendReleaseComplete(
       H323SignalPDU & releaseCompletePDU ///<  Release Complete PDU to send
@@ -910,7 +897,7 @@ class H323Connection : public OpalRTPConnection
        receives the optional Alerting PDU from the remote endpoint. That is
        when the remote "phone" is "ringing".
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour calls the endpoint function of the same name.
@@ -929,10 +916,10 @@ class H323Connection : public OpalRTPConnection
        but only indicate to whatever other thread is gathering digits that
        more are required and that thread should call SendMoreDigits().
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PFalse.
+       The default behaviour simply returns false.
      */
     virtual PBoolean OnInsufficientDigits();
 
@@ -943,10 +930,10 @@ class H323Connection : public OpalRTPConnection
        The digits parameter is appended to the existing remoteNumber member
        variable and the call is retried.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual void SendMoreDigits(
       const PString & digits    ///<  Extra digits
@@ -956,7 +943,7 @@ class H323Connection : public OpalRTPConnection
        receives the Connect PDU from the remote endpoint, but before it
        attempts to open the control channel.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour calls H323EndPoint::OnOutgoingCall
@@ -970,7 +957,7 @@ class H323Connection : public OpalRTPConnection
        connection by the original SETUP PDU have been selected and opened and
        need to be sent back to the remote endpoint.
 
-       If PFalse is returned then no fast start has been acknowledged, possibly
+       If false is returned then no fast start has been acknowledged, possibly
        due to no common codec in fast start request.
 
        The default behaviour uses OnSelectLogicalChannels() to find a pair of
@@ -985,7 +972,7 @@ class H323Connection : public OpalRTPConnection
        receives a PDU from the remote endpoint that has a fastStart field. It
        is in response to a request for a fast strart from the local endpoint.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour parses the provided array and starts the channels
@@ -1002,7 +989,7 @@ class H323Connection : public OpalRTPConnection
        This function is called from one of a number of functions after it
        receives a PDU from the remote endpoint that has a h245Address field.
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks to see if it is a known transport and
@@ -1018,7 +1005,7 @@ class H323Connection : public OpalRTPConnection
        to listen for an incoming H.245 connection.
 
 
-       If PFalse is returned the connection is aborted and a Release Complete
+       If false is returned the connection is aborted and a Release Complete
        PDU is sent.
 
        The default behaviour checks to see if it is a known transport and
@@ -1048,8 +1035,8 @@ class H323Connection : public OpalRTPConnection
     /**Handle incoming data on the control channel.
        This decodes the data stream into a PDU and calls HandleControlPDU().
 
-       If PFalse is returned the connection is aborted. The default behaviour
-       returns PTrue.
+       If false is returned the connection is aborted. The default behaviour
+       returns true.
      */
     virtual PBoolean HandleControlData(
       PPER_Stream & strm
@@ -1058,8 +1045,8 @@ class H323Connection : public OpalRTPConnection
     /**Handle incoming PDU's on the control channel. Dispatches them to the
        various virtuals off this class.
 
-       If PFalse is returned the connection is aborted. The default behaviour
-       returns PTrue.
+       If false is returned the connection is aborted. The default behaviour
+       returns true.
      */
     virtual PBoolean HandleControlPDU(
       const H323ControlPDU & pdu
@@ -1068,11 +1055,11 @@ class H323Connection : public OpalRTPConnection
     /**This function is called from the HandleControlPDU() function
        for unhandled PDU types.
 
-       If PFalse is returned the connection is aborted and a Release Complete
-       PDU is sent. The default behaviour returns PTrue.
+       If false is returned the connection is aborted and a Release Complete
+       PDU is sent. The default behaviour returns true.
 
        The default behaviour send a FunctioNotUnderstood indication back to
-       the sender, and returns PTrue to continue operation.
+       the sender, and returns true to continue operation.
      */
     virtual PBoolean OnUnknownControlPDU(
       const H323ControlPDU & pdu  ///<  Received PDU
@@ -1177,6 +1164,17 @@ class H323Connection : public OpalRTPConnection
       unsigned terminalLabel
     );
 
+    /**Send a H.239 presentation token request.
+       Asks the remote for the "token" allow local system to own the
+       presentation video channel. A OnH239PresentationResponse() callback
+       should occur when the remote responds to the request.
+      */
+    virtual bool SendH239PresentationRequest(
+      unsigned logicalChannel,
+      unsigned symmetryBreaking,
+      unsigned terminalLabel
+    );
+
     /**Handle a H.239 presentation token ack/reject response.
        Default behaviour simply sends a release command.
       */
@@ -1194,6 +1192,15 @@ class H323Connection : public OpalRTPConnection
       unsigned terminalLabel
     );
 
+    /**Send a H.239 presentation token release.
+       Tell the remote we no longer require the "token" allowing the local
+       system to own the presentation video channel.
+      */
+    virtual bool SendH239PresentationRelease(
+      unsigned logicalChannel,
+      unsigned terminalLabel
+    );
+
     /**Handle a H.239 presentation token owner indication.
        Default behaviour does nothing.
       */
@@ -1221,8 +1228,8 @@ class H323Connection : public OpalRTPConnection
           e_UnhandledPDU                    &H323ControlPDU
           e_MasterSlaveDetermination        const char *
 
-       If PFalse is returned the connection is aborted. The default behaviour
-       returns PTrue.
+       If false is returned the connection is aborted. The default behaviour
+       returns true.
      */
     virtual PBoolean OnControlProtocolError(
       ControlProtocolErrors errorSource,  ///<  Source of the proptoerror
@@ -1249,12 +1256,12 @@ class H323Connection : public OpalRTPConnection
        included in the remoteCodecs list.
 
        The default behaviour assigns the table and set to member variables and
-       returns PTrue if the remoteCodecs list is not empty.
+       returns true if the remoteCodecs list is not empty.
      */
     virtual PBoolean OnReceivedCapabilitySet(
       const H323Capabilities & remoteCaps,      ///<  Capability combinations remote supports
       const H245_MultiplexCapability * muxCap,  ///<  Transport capability, if present
-      H245_TerminalCapabilitySetReject & reject ///<  Rejection PDU (if return PFalse)
+      H245_TerminalCapabilitySetReject & reject ///<  Rejection PDU (if return false)
     );
 
     /**Send a new capability set.
@@ -1367,7 +1374,7 @@ class H323Connection : public OpalRTPConnection
 
        If this function is called later in the call sequence, eg from
        OnSelectLogicalChannels(), then it may only establish a transmit
-       channel, ie fromRemote must be PFalse.
+       channel, ie fromRemote must be false.
       */
     virtual PBoolean OpenLogicalChannel(
       const H323Capability & capability,  ///<  Capability to open channel with
@@ -1383,11 +1390,11 @@ class H323Connection : public OpalRTPConnection
     /**This function is called when the remote endpoint want's to open
        a new channel.
 
-       If the return value is PFalse then the open is rejected using the
+       If the return value is false then the open is rejected using the
        errorCode as the cause, this would be a value from the enum
        H245_OpenLogicalChannelReject_cause::Choices.
 
-       The default behaviour simply returns PTrue.
+       The default behaviour simply returns true.
      */
     virtual PBoolean OnOpenLogicalChannel(
       const H245_OpenLogicalChannel & openPDU,  ///<  Received PDU for the channel open
@@ -1471,7 +1478,7 @@ class H323Connection : public OpalRTPConnection
     /**This function is called when the remote endpoint want's to create
        a new channel.
 
-       If the return value is PFalse then the open is rejected using the
+       If the return value is false then the open is rejected using the
        errorCode as the cause, this would be a value from the enum
        H245_OpenLogicalChannelReject_cause::Choices.
 
@@ -1486,7 +1493,7 @@ class H323Connection : public OpalRTPConnection
 
     /**Call back function when a logical channel thread begins.
 
-       The default behaviour does nothing and returns PTrue.
+       The default behaviour does nothing and returns true.
       */
     virtual PBoolean OnStartLogicalChannel(
       H323Channel & channel    ///<  Channel that has been started.
@@ -1524,9 +1531,9 @@ class H323Connection : public OpalRTPConnection
        a logical channel.
 
        The application may get an opportunity to refuse to close the channel by
-       returning PFalse from this function.
+       returning false from this function.
 
-       The default behaviour returns PTrue.
+       The default behaviour returns true.
      */
     virtual PBoolean OnClosingLogicalChannel(
       H323Channel & channel   ///<  Channel that is to be closed
@@ -1582,12 +1589,12 @@ class H323Connection : public OpalRTPConnection
   /**@name Bandwidth Management */
   //@{
     /**Set the available bandwidth in 100's of bits/sec.
-       Note if the force parameter is PTrue this function will close down
+       Note if the force parameter is true this function will close down
        active logical channels to meet the new bandwidth requirement.
       */
     virtual PBoolean SetBandwidthAvailable(
       unsigned newBandwidth,    ///<  New bandwidth limit
-      PBoolean force = PFalse        ///<  Force bandwidth limit
+      PBoolean force = false        ///<  Force bandwidth limit
     );
 
     /**Get the bandwidth currently used.
@@ -1717,10 +1724,6 @@ class H323Connection : public OpalRTPConnection
        If there is no session of the specified ID, a new one is created using
        the information provided in the tranport parameter. If the system
        does not support the specified transport, NULL is returned.
-
-       If this function is used, then the ReleaseSession() function MUST be
-       called or the session is never deleted for the lifetime of the H323
-       connection.
       */
     virtual RTP_Session * UseSession(
       const OpalTransport & transport,
@@ -1729,13 +1732,6 @@ class H323Connection : public OpalRTPConnection
                   RTP_QOS * rtpqos = NULL
     );
 
-    /**Release the session. If the session ID is not being used any more any
-       clients via the UseSession() function, then the session is deleted.
-     */
-    virtual void ReleaseSession(
-      unsigned sessionID
-    );
-
     /**Callback from the RTP session for statistics monitoring.
        This is called every so many packets on the transmitter and receiver
        threads of the RTP session indicating that the statistics have been
@@ -1770,7 +1766,7 @@ class H323Connection : public OpalRTPConnection
        remote should either start sending G.723 and H.261, G.729 and H.261 or
        just G.728 on its own.
 
-       Returns PFalse if a mode change is currently in progress, only one mode
+       Returns false if a mode change is currently in progress, only one mode
        change may be done at a time.
       */
     virtual PBoolean RequestModeChange(
@@ -1781,7 +1777,7 @@ class H323Connection : public OpalRTPConnection
        This asks the remote system to stop it transmitters and start sending
        one of the combinations specifed.
 
-       Returns PFalse if a mode change is currently in progress, only one mode
+       Returns false if a mode change is currently in progress, only one mode
        change may be done at a time.
       */
     virtual PBoolean RequestModeChange(
@@ -1839,10 +1835,10 @@ class H323Connection : public OpalRTPConnection
        This allows an individual ARQ to override the authentical credentials
        used in H.235 based RAS for this particular connection.
 
-       A return value of PFalse indicates to use the default credentials of the
-       endpoint, while PTrue indicates that new credentials are to be used.
+       A return value of false indicates to use the default credentials of the
+       endpoint, while true indicates that new credentials are to be used.
 
-       The default behavour does nothing and returns PFalse.
+       The default behavour does nothing and returns false.
       */
     virtual PBoolean GetAdmissionRequestAuthentication(
       const H225_AdmissionRequest & arq,  ///<  ARQ being constructed
@@ -1920,12 +1916,6 @@ class H323Connection : public OpalRTPConnection
       const H225_EndpointType & pdu ///<  PDU from which to extract application info.
     );
     
-    /**Get the remote party address.
-       This will return the "best guess" at an address to use in a
-       to call the user again later.
-      */
-    PString GetRemotePartyURL() const;
-    
     /**Get the remotes capability table for this connection.
      */
     const H323Capabilities & GetLocalCapabilities() const { return localCapabilities; }
@@ -2043,7 +2033,15 @@ class H323Connection : public OpalRTPConnection
     H4507Handler&  getH4507handler(){return *h4507handler;}
 #endif
 
-    virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    /** Callback for media commands.
+        Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
+      */
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
     
   protected:
     /**Internal function to check if call established.
@@ -2054,6 +2052,7 @@ class H323Connection : public OpalRTPConnection
     virtual void InternalEstablishedConnectionCheck();
     PBoolean InternalEndSessionCheck(PPER_Stream & strm);
     void SetRemoteVersions(const H225_ProtocolIdentifier & id);
+    void SetBearerCapabilities(H323SignalPDU & pdu) const;
     void MonitorCallStatus();
     PDECLARE_NOTIFIER(PThread, H323Connection, StartOutgoing);
     PDECLARE_NOTIFIER(PThread, H323Connection, NewOutgoingControlChannel);
@@ -2063,9 +2062,10 @@ class H323Connection : public OpalRTPConnection
 
     H323TransportAddress m_remoteConnectAddress;
     int                  remoteCallWaiting; // Number of call's waiting at the remote endpoint
-    PBoolean                 gatekeeperRouted;
+    bool                 gatekeeperRouted;
     unsigned             distinctiveRing;
     unsigned             callReference;
+    unsigned             m_progressIndicator;
     OpalGloballyUniqueID callIdentifier;
     OpalGloballyUniqueID conferenceIdentifier;
 
@@ -2076,16 +2076,15 @@ class H323Connection : public OpalRTPConnection
     H323Capabilities   remoteCapabilities; // Capabilities remote system supports
     unsigned           remoteMaxAudioDelayJitter;
     PTimer             roundTripDelayTimer;
-    WORD               maxAudioDelayJitter;
     unsigned           uuiesRequested;
     PString            gkAccessTokenOID;
     PBYTEArray         gkAccessTokenData;
-    PBoolean               addAccessTokenToSetup;
+    bool               addAccessTokenToSetup;
 
     H323Transport * signallingChannel;
     H323Transport * controlChannel;
     OpalListener  * controlListener;
-    PBoolean            h245Tunneling;
+    bool            h245Tunneling;
     H323SignalPDU * h245TunnelRxPDU;
     H323SignalPDU * h245TunnelTxPDU;
     H323SignalPDU * setupPDU;
@@ -2107,27 +2106,27 @@ class H323Connection : public OpalRTPConnection
 
     unsigned   h225version;
     unsigned   h245version;
-    PBoolean       h245versionSet;
-    PBoolean doH245inSETUP;
-    PBoolean lastPDUWasH245inSETUP;
+    bool       h245versionSet;
+    bool       doH245inSETUP;
+    bool       lastPDUWasH245inSETUP;
 
-    PBoolean mustSendDRQ;
-    PBoolean mediaWaitForConnect;
-    PBoolean transmitterSidePaused;
-    bool     remoteTransmitPaused;
-    PBoolean earlyStart;
+    bool       mustSendDRQ;
+    bool       mediaWaitForConnect;
+    bool       transmitterSidePaused;
+    bool       remoteTransmitPaused;
+    bool       earlyStart;
     PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
-    PBoolean       endSessionNeeded;
+    bool       endSessionNeeded;
     PSyncPoint endSessionReceived;
     PTimer     enforcedDurationLimit;
 
     // Used as part of a local call hold operation involving MOH
     PChannel * holdMediaChannel;
-    PBoolean       isConsultationTransfer;
+    bool       isConsultationTransfer;
 
     /** Call Intrusion flag and parameters */
-    PBoolean     isCallIntrusion;
+    bool     isCallIntrusion;
     unsigned callIntrusionProtectionLevel;
 
     enum FastStartStates {
@@ -2176,6 +2175,9 @@ class H323Connection : public OpalRTPConnection
 
   private:
     PChannel * SwapHoldMediaChannels(PChannel * newChannel);
+
+    P_REMOVE_VIRTUAL_VOID(CleanUpOnCallEnd());
+    P_REMOVE_VIRTUAL_VOID(OnCleared());
 };
 
 
