@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25556 $
+ * $Revision: 28276 $
  * $Author: rjongbloed $
- * $Date: 2011-04-13 04:35:30 -0500 (Wed, 13 Apr 2011) $
+ * $Date: 2012-08-29 21:50:30 -0500 (Wed, 29 Aug 2012) $
  */
 
 #include <ptlib.h>
@@ -449,6 +449,48 @@ PBoolean OpalPCSSConnection::SetAudioVolume(PBoolean source, unsigned percentage
 }
 
 
+PBoolean OpalPCSSConnection::GetAudioVolume(PBoolean source, unsigned & percentage)
+{
+  PSafePtr<OpalAudioMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalAudioMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
+  if (stream == NULL)
+    return PFalse;
+
+  PSoundChannel * channel = dynamic_cast<PSoundChannel *>(stream->GetChannel());
+  if (channel == NULL)
+    return PFalse;
+
+  return channel->GetVolume(percentage);
+}
+
+
+bool OpalPCSSConnection::SetAudioMute(bool source, bool mute)
+{
+  PSafePtr<OpalAudioMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalAudioMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
+  if (stream == NULL)
+    return PFalse;
+
+  PSoundChannel * channel = dynamic_cast<PSoundChannel *>(stream->GetChannel());
+  if (channel == NULL)
+    return PFalse;
+
+  return channel->SetMute(mute);
+}
+
+
+bool OpalPCSSConnection::GetAudioMute(bool source, bool & mute)
+{
+  PSafePtr<OpalAudioMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalAudioMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
+  if (stream == NULL)
+    return PFalse;
+
+  PSoundChannel * channel = dynamic_cast<PSoundChannel *>(stream->GetChannel());
+  if (channel == NULL)
+    return PFalse;
+
+  return channel->GetMute(mute);
+}
+
+
 unsigned OpalPCSSConnection::GetAudioSignalLevel(PBoolean source)
 {
   PSafePtr<OpalAudioMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalAudioMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
